@@ -3,8 +3,9 @@
 # pylint: disable=too-few-public-methods
 
 import json
+import sys
+import warnings
 from typing import Any, Dict, List, Optional
-from uuid import uuid4
 
 import httpx
 import pydantic
@@ -13,6 +14,19 @@ from starlette.exceptions import HTTPException
 from starlette.requests import Request
 from starlette.responses import RedirectResponse
 
+if sys.version_info >= (3, 8):
+    from typing import TypedDict
+else:
+    from typing_extensions import TypedDict
+
+DiscoveryDocument = TypedDict(
+    "DiscoveryDocument", {"authorization_endpoint": str, "token_endpoint": str, "userinfo_endpoint": str}
+)
+
+
+class UnsetStateWarning(UserWarning):
+    """Warning about unset state parameter"""
+
 
 class SSOLoginError(HTTPException):
     """Raised when any login-related error ocurrs
@@ -23,13 +37,13 @@ class SSOLoginError(HTTPException):
 class OpenID(pydantic.BaseModel):  # pylint: disable=no-member
     """Class (schema) to represent information got from sso provider in a common form."""
 
-    id: Optional[str]
-    email: Optional[str]
-    first_name: Optional[str]
-    last_name: Optional[str]
-    display_name: Optional[str]
-    picture: Optional[str]
-    provider: Optional[str]
+    id: Optional[str] = None
+    email: Optional[str] = None
+    first_name: Optional[str] = None
+    last_name: Optional[str] = None
+    display_name: Optional[str] = None
+    picture: Optional[str] = None
+    provider: Optional[str] = None
 
 
 # pylint: disable=too-many-instance-attributes
@@ -42,7 +56,7 @@ class SSOBase:
     redirect_uri: Optional[str] = NotImplemented
     scope: List[str] = NotImplemented
     _oauth_client: Optional[WebApplicationClient] = None
-    state: Optional[str] = None
+    additional_headers: Optional[Dict[str, Any]] = None
 
     def __init__(
         self,
@@ -50,7 +64,7 @@ class SSOBase:
         client_secret: str,
         redirect_uri: Optional[str] = None,
         allow_insecure_http: bool = False,
-        use_state: bool = True,
+        use_state: bool = False,
         scope: Optional[List[str]] = None,
     ):
         # pylint: disable=too-many-arguments
@@ -58,9 +72,29 @@ class SSOBase:
         self.client_secret = client_secret
         self.redirect_uri = redirect_uri
         self.allow_insecure_http = allow_insecure_http
-        self.use_state = use_state
+        # TODO: Remove use_state argument and attribute
+        if use_state:
+            warnings.warn(
+                (
+                    "Argument 'use_state' of SSOBase's constructor is deprecated and will be removed in "
+                    "future releases. Use 'state' argument of individual methods instead."
+                ),
+                DeprecationWarning,
+            )
         self.scope = scope or self.scope
         self._refresh_token: Optional[str] = None
+        self._state: Optional[str] = None
+
+    @property
+    def state(self) -> Optional[str]:
+        """Gets state as it was returned from the server"""
+        if self._state is None:
+            warnings.warn(
+                "'state' parameter is unset. This means the server either "
+                "didn't return state (was this expected?) or 'verify_and_process' hasn't been called yet.",
+                UnsetStateWarning,
+            )
+        return self._state
 
     @property
     def oauth_client(self) -> WebApplicationClient:
@@ -86,7 +120,7 @@ class SSOBase:
         """Return {OpenID} object from provider's user info endpoint response"""
         raise NotImplementedError(f"Provider {cls.provider} not supported")
 
-    async def get_discovery_document(self) -> Dict[str, str]:
+    async def get_discovery_document(self) -> DiscoveryDocument:
         """Get discovery document containing handy urls"""
         raise NotImplementedError(f"Provider {self.provider} not supported")
 
@@ -109,40 +143,51 @@ class SSOBase:
         return discovery.get("userinfo_endpoint")
 
     async def get_login_url(
-        self, *, redirect_uri: Optional[str] = None, params: Optional[Dict[str, Any]] = None
+        self,
+        *,
+        redirect_uri: Optional[str] = None,
+        params: Optional[Dict[str, Any]] = None,
+        state: Optional[str] = None,
     ) -> str:
         """Return prepared login url. This is low-level, see {get_login_redirect} instead."""
         params = params or {}
         redirect_uri = redirect_uri or self.redirect_uri
         if redirect_uri is None:
             raise ValueError("redirect_uri must be provided, either at construction or request time")
-        if self.use_state:
-            self.state = str(uuid4())
         request_uri = self.oauth_client.prepare_request_uri(
-            await self.authorization_endpoint, redirect_uri=redirect_uri, state=self.state, scope=self.scope, **params
+            await self.authorization_endpoint, redirect_uri=redirect_uri, state=state, scope=self.scope, **params
         )
         return request_uri
 
     async def get_login_redirect(
-        self, *, redirect_uri: Optional[str] = None, params: Optional[Dict[str, Any]] = None
+        self,
+        *,
+        redirect_uri: Optional[str] = None,
+        params: Optional[Dict[str, Any]] = None,
+        state: Optional[str] = None,
     ) -> RedirectResponse:
         """Return redirect response by Stalette to login page of Oauth SSO provider
 
         Arguments:
             redirect_uri {Optional[str]} -- Override redirect_uri specified on this instance (default: None)
             params {Optional[Dict[str, Any]]} -- Add additional query parameters to the login request.
+            state {Optional[str]} -- Add state parameter. This is useful if you want
+                                    the server to return something specific back to you.
 
         Returns:
             RedirectResponse -- Starlette response (may directly be returned from FastAPI)
         """
-        login_uri = await self.get_login_url(redirect_uri=redirect_uri, params=params)
+        login_uri = await self.get_login_url(redirect_uri=redirect_uri, params=params, state=state)
         response = RedirectResponse(login_uri, 303)
-        if self.state is not None and self.use_state:
-            response.set_cookie("ssostate", self.state, expires=600)
         return response
 
     async def verify_and_process(
-        self, request: Request, *, params: Optional[Dict[str, Any]] = None
+        self,
+        request: Request,
+        *,
+        params: Optional[Dict[str, Any]] = None,
+        headers: Optional[Dict[str, Any]] = None,
+        redirect_uri: Optional[str] = None,
     ) -> Optional[OpenID]:
         """Get FastAPI (Starlette) Request object and process login.
         This handler should be used for your /callback path.
@@ -154,30 +199,35 @@ class SSOBase:
         Returns:
             Optional[OpenID] -- OpenID if the login was successfull
         """
+        headers = headers or {}
         code = request.query_params.get("code")
         if code is None:
             raise SSOLoginError(400, "'code' parameter was not found in callback request")
-        if self.state is not None and self.use_state:
-            ssostate = request.cookies.get("ssostate")
-            if ssostate is None or ssostate != self.state:
-                raise SSOLoginError(
-                    401,
-                    "'state' parameter in callback request does not match our internal 'state', "
-                    "someone may be trying to do something bad.",
-                )
-        return await self.process_login(code, request, params=params)
+        self._state = request.query_params.get("state")
+        return await self.process_login(
+            code, request, params=params, additional_headers=headers, redirect_uri=redirect_uri
+        )
 
     async def process_login(
-        self, code: str, request: Request, *, params: Optional[Dict[str, Any]] = None
+        self,
+        code: str,
+        request: Request,
+        *,
+        params: Optional[Dict[str, Any]] = None,
+        additional_headers: Optional[Dict[str, Any]] = None,
+        redirect_uri: Optional[str] = None,
     ) -> Optional[OpenID]:
         """This method should be called from callback endpoint to verify the user and request user info endpoint.
         This is low level, you should use {verify_and_process} instead.
 
         Arguments:
             params {Optional[Dict[str, Any]]} -- Optional additional query parameters to pass to the provider
+            additional_headers {Optional[Dict[str, Any]]} -- Optional additional headers to be added to all requests
         """
         # pylint: disable=too-many-locals
         params = params or {}
+        additional_headers = additional_headers or {}
+        additional_headers.update(self.additional_headers or {})
         url = request.url
         scheme = url.scheme
         if not self.allow_insecure_http and scheme != "https":
@@ -190,7 +240,7 @@ class SSOBase:
         token_url, headers, body = self.oauth_client.prepare_token_request(
             await self.token_endpoint,
             authorization_response=current_url,
-            redirect_url=current_path,
+            redirect_url=redirect_uri or self.redirect_uri or current_path,
             code=code,
             **params,
         )  # type: ignore
@@ -198,6 +248,8 @@ class SSOBase:
         if token_url is None:
             return None
 
+        headers.update(additional_headers)
+
         auth = httpx.BasicAuth(self.client_id, self.client_secret)
         async with httpx.AsyncClient() as session:
             response = await session.post(token_url, headers=headers, content=body, auth=auth)
