@@ -1,8 +1,8 @@
 """Microsoft SSO Oauth Helper class"""
 
-from typing import Dict, List, Optional
+from typing import List, Optional
 
-from fastapi_sso.sso.base import OpenID, SSOBase
+from fastapi_sso.sso.base import DiscoveryDocument, OpenID, SSOBase
 
 
 class MicrosoftSSO(SSOBase):
@@ -19,7 +19,7 @@ class MicrosoftSSO(SSOBase):
         client_secret: str,
         redirect_uri: Optional[str] = None,
         allow_insecure_http: bool = False,
-        use_state: bool = True,
+        use_state: bool = False,  # TODO: Remove use_state argument
         scope: Optional[List[str]] = None,
         tenant: Optional[str] = None,
     ):
@@ -28,12 +28,12 @@ class MicrosoftSSO(SSOBase):
             client_secret=client_secret,
             redirect_uri=redirect_uri,
             allow_insecure_http=allow_insecure_http,
-            use_state=use_state,
+            use_state=use_state,  # TODO: Remove use_state argument
             scope=scope,
         )
         self.tenant = tenant or self.tenant
 
-    async def get_discovery_document(self) -> Dict[str, str]:
+    async def get_discovery_document(self) -> DiscoveryDocument:
         return {
             "authorization_endpoint": f"https://login.microsoftonline.com/{self.tenant}/oauth2/v2.0/authorize",
             "token_endpoint": f"https://login.microsoftonline.com/{self.tenant}/oauth2/v2.0/token",
